@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # BSD 2-Clause License
 #
 # Apprise - Push Notification Library.
@@ -46,49 +45,50 @@
 #  - https://sendgrid.com/docs/ui/sending-email/\
 #       how-to-send-an-email-with-dynamic-transactional-templates/
 
-import requests
 from json import dumps
 
-from .base import NotifyBase
-from .. import exception
-from ..common import NotifyFormat
-from ..common import NotifyType
-from ..utils.parse import parse_list, is_email, validate_regex
-from ..locale import gettext_lazy as _
+import requests
 
+from .. import exception
+from ..common import NotifyFormat, NotifyType
+from ..locale import gettext_lazy as _
+from ..utils.parse import is_email, parse_list, validate_regex
+from .base import NotifyBase
 
 # Extend HTTP Error Messages
 SENDGRID_HTTP_ERROR_MAP = {
-    401: 'Unauthorized - You do not have authorization to make the request.',
-    413: 'Payload To Large - The JSON payload you have included in your '
-         'request is too large.',
-    429: 'Too Many Requests - The number of requests you have made exceeds '
-         'SendGrid’s rate limitations.',
+    401: "Unauthorized - You do not have authorization to make the request.",
+    413: (
+        "Payload To Large - The JSON payload you have included in your "
+        "request is too large."
+    ),
+    429: (
+        "Too Many Requests - The number of requests you have made exceeds "
+        "SendGrid's rate limitations."
+    ),
 }
 
 
 class NotifySendGrid(NotifyBase):
-    """
-    A wrapper for Notify SendGrid Notifications
-    """
+    """A wrapper for Notify SendGrid Notifications."""
 
     # The default descriptive name associated with the Notification
-    service_name = 'SendGrid'
+    service_name = "SendGrid"
 
     # The services URL
-    service_url = 'https://sendgrid.com'
+    service_url = "https://sendgrid.com"
 
     # The default secure protocol
-    secure_protocol = 'sendgrid'
+    secure_protocol = "sendgrid"
 
     # A URL that takes you to the setup/help of the specific protocol
-    setup_url = 'https://github.com/caronc/apprise/wiki/Notify_sendgrid'
+    setup_url = "https://github.com/caronc/apprise/wiki/Notify_sendgrid"
 
     # Default to markdown
     notify_format = NotifyFormat.HTML
 
     # The default Email API URL to use
-    notify_url = 'https://api.sendgrid.com/v3/mail/send'
+    notify_url = "https://api.sendgrid.com/v3/mail/send"
 
     # Support attachments
     attachment_support = True
@@ -98,95 +98,108 @@ class NotifySendGrid(NotifyBase):
     request_rate_per_sec = 0.2
 
     # The default subject to use if one isn't specified.
-    default_empty_subject = '<no subject>'
+    default_empty_subject = "<no subject>"
 
     # Define object templates
     templates = (
-        '{schema}://{apikey}:{from_email}',
-        '{schema}://{apikey}:{from_email}/{targets}',
+        "{schema}://{apikey}:{from_email}",
+        "{schema}://{apikey}:{from_email}/{targets}",
     )
 
     # Define our template arguments
-    template_tokens = dict(NotifyBase.template_tokens, **{
-        'apikey': {
-            'name': _('API Key'),
-            'type': 'string',
-            'private': True,
-            'required': True,
-            'regex': (r'^[A-Z0-9._-]+$', 'i'),
+    template_tokens = dict(
+        NotifyBase.template_tokens,
+        **{
+            "apikey": {
+                "name": _("API Key"),
+                "type": "string",
+                "private": True,
+                "required": True,
+                "regex": (r"^[A-Z0-9._-]+$", "i"),
+            },
+            "from_email": {
+                "name": _("Source Email"),
+                "type": "string",
+                "required": True,
+            },
+            "target_email": {
+                "name": _("Target Email"),
+                "type": "string",
+                "map_to": "targets",
+            },
+            "targets": {
+                "name": _("Targets"),
+                "type": "list:string",
+            },
         },
-        'from_email': {
-            'name': _('Source Email'),
-            'type': 'string',
-            'required': True,
-        },
-        'target_email': {
-            'name': _('Target Email'),
-            'type': 'string',
-            'map_to': 'targets',
-        },
-        'targets': {
-            'name': _('Targets'),
-            'type': 'list:string',
-        },
-    })
+    )
 
     # Define our template arguments
-    template_args = dict(NotifyBase.template_args, **{
-        'to': {
-            'alias_of': 'targets',
+    template_args = dict(
+        NotifyBase.template_args,
+        **{
+            "to": {
+                "alias_of": "targets",
+            },
+            "cc": {
+                "name": _("Carbon Copy"),
+                "type": "list:string",
+            },
+            "bcc": {
+                "name": _("Blind Carbon Copy"),
+                "type": "list:string",
+            },
+            "template": {
+                # Template ID
+                # The template ID is 64 characters with one dash (d-uuid)
+                "name": _("Template"),
+                "type": "string",
+            },
         },
-        'cc': {
-            'name': _('Carbon Copy'),
-            'type': 'list:string',
-        },
-        'bcc': {
-            'name': _('Blind Carbon Copy'),
-            'type': 'list:string',
-        },
-        'template': {
-            # Template ID
-            # The template ID is 64 characters with one dash (d-uuid)
-            'name': _('Template'),
-            'type': 'string',
-        },
-    })
+    )
 
     # Support Template Dynamic Variables (Substitutions)
     template_kwargs = {
-        'template_data': {
-            'name': _('Template Data'),
-            'prefix': '+',
+        "template_data": {
+            "name": _("Template Data"),
+            "prefix": "+",
         },
     }
 
-    def __init__(self, apikey, from_email, targets=None, cc=None,
-                 bcc=None, template=None, template_data=None, **kwargs):
-        """
-        Initialize Notify SendGrid Object
-        """
+    def __init__(
+        self,
+        apikey,
+        from_email,
+        targets=None,
+        cc=None,
+        bcc=None,
+        template=None,
+        template_data=None,
+        **kwargs,
+    ):
+        """Initialize Notify SendGrid Object."""
         super().__init__(**kwargs)
 
         # API Key (associated with project)
         self.apikey = validate_regex(
-            apikey, *self.template_tokens['apikey']['regex'])
+            apikey, *self.template_tokens["apikey"]["regex"]
+        )
         if not self.apikey:
-            msg = 'An invalid SendGrid API Key ' \
-                  '({}) was specified.'.format(apikey)
+            msg = f"An invalid SendGrid API Key ({apikey}) was specified."
             self.logger.warning(msg)
             raise TypeError(msg)
 
         result = is_email(from_email)
         if not result:
-            msg = 'Invalid ~From~ email specified: {}'.format(from_email)
+            msg = f"Invalid ~From~ email specified: {from_email}"
             self.logger.warning(msg)
             raise TypeError(msg)
 
         # Store email address
-        self.from_email = result['full_email']
+        self.from_email = result["full_email"]
 
         # Acquire Targets (To Emails)
-        self.targets = list()
+        self.targets = []
 
         # Acquire Carbon Copies
         self.cc = set()
@@ -198,20 +211,20 @@ class NotifySendGrid(NotifyBase):
         self.template = template
 
         # Now our dynamic template data (if defined)
-        self.template_data = template_data \
-            if isinstance(template_data, dict) else {}
+        self.template_data = (
+            template_data if isinstance(template_data, dict) else {}
+        )
 
         # Validate recipients (to:) and drop bad ones:
         for recipient in parse_list(targets):
 
             result = is_email(recipient)
             if result:
-                self.targets.append(result['full_email'])
+                self.targets.append(result["full_email"])
                 continue
 
             self.logger.warning(
-                'Dropped invalid email '
-                '({}) specified.'.format(recipient),
+                f"Dropped invalid email ({recipient}) specified.",
             )
 
         # Validate recipients (cc:) and drop bad ones:
@@ -219,12 +232,11 @@ class NotifySendGrid(NotifyBase):
 
             result = is_email(recipient)
             if result:
-                self.cc.add(result['full_email'])
+                self.cc.add(result["full_email"])
                 continue
 
             self.logger.warning(
-                'Dropped invalid Carbon Copy email '
-                '({}) specified.'.format(recipient),
+                f"Dropped invalid Carbon Copy email ({recipient}) specified.",
             )
 
         # Validate recipients (bcc:) and drop bad ones:
@@ -232,12 +244,12 @@ class NotifySendGrid(NotifyBase):
 
             result = is_email(recipient)
             if result:
-                self.bcc.add(result['full_email'])
+                self.bcc.add(result["full_email"])
                 continue
 
             self.logger.warning(
-                'Dropped invalid Blind Carbon Copy email '
-                '({}) specified.'.format(recipient),
+                "Dropped invalid Blind Carbon Copy email "
+                f"({recipient}) specified.",
             )
 
         if len(self.targets) == 0:
@@ -248,68 +260,73 @@ class NotifySendGrid(NotifyBase):
 
     @property
     def url_identifier(self):
-        """
-        Returns all of the identifiers that make this URL unique from
-        another simliar one. Targets or end points should never be identified
-        here.
+        """Returns all of the identifiers that make this URL unique from
+        another simliar one.
+
+        Targets or end points should never be identified here.
         """
         return (self.secure_protocol, self.apikey, self.from_email)
 
     def url(self, privacy=False, *args, **kwargs):
-        """
-        Returns the URL built dynamically based on specified arguments.
-        """
+        """Returns the URL built dynamically based on specified arguments."""
 
         # Our URL parameters
         params = self.url_parameters(privacy=privacy, *args, **kwargs)
 
         if len(self.cc) > 0:
             # Handle our Carbon Copy Addresses
-            params['cc'] = ','.join(self.cc)
+            params["cc"] = ",".join(self.cc)
 
         if len(self.bcc) > 0:
             # Handle our Blind Carbon Copy Addresses
-            params['bcc'] = ','.join(self.bcc)
+            params["bcc"] = ",".join(self.bcc)
 
         if self.template:
             # Handle our Template ID if if was specified
-            params['template'] = self.template
+            params["template"] = self.template
 
         # Append our template_data into our parameter list
-        params.update(
-            {'+{}'.format(k): v for k, v in self.template_data.items()})
+        params.update({f"+{k}": v for k, v in self.template_data.items()})
 
         # a simple boolean check as to whether we display our target emails
         # or not
-        has_targets = \
-            not (len(self.targets) == 1 and self.targets[0] == self.from_email)
+        has_targets = not (
+            len(self.targets) == 1 and self.targets[0] == self.from_email
+        )
 
-        return '{schema}://{apikey}:{from_email}/{targets}?{params}'.format(
+        return "{schema}://{apikey}:{from_email}/{targets}?{params}".format(
             schema=self.secure_protocol,
-            apikey=self.pprint(self.apikey, privacy, safe=''),
+            apikey=self.pprint(self.apikey, privacy, safe=""),
             # never encode email since it plays a huge role in our hostname
             from_email=self.from_email,
-            targets='' if not has_targets else '/'.join(
-                [NotifySendGrid.quote(x, safe='') for x in self.targets]),
+            targets=(
+                ""
+                if not has_targets
+                else "/".join(
+                    [NotifySendGrid.quote(x, safe="") for x in self.targets]
+                )
+            ),
             params=NotifySendGrid.urlencode(params),
         )
 
     def __len__(self):
-        """
-        Returns the number of targets associated with this notification
-        """
+        """Returns the number of targets associated with this notification."""
         return len(self.targets)
 
-    def send(self, body, title='', notify_type=NotifyType.INFO, attach=None,
-             **kwargs):
-        """
-        Perform SendGrid Notification
-        """
+    def send(
+        self,
+        body,
+        title="",
+        notify_type=NotifyType.INFO,
+        attach=None,
+        **kwargs,
+    ):
+        """Perform SendGrid Notification."""
 
         headers = {
-            'User-Agent': self.app_id,
-            'Content-Type': 'application/json',
-            'Authorization': 'Bearer {}'.format(self.apikey),
+            "User-Agent": self.app_id,
+            "Content-Type": "application/json",
+            "Authorization": f"Bearer {self.apikey}",
         }
 
         # error tracking (used for function return)
@@ -317,21 +334,24 @@ class NotifySendGrid(NotifyBase):
 
         # A Simple Email Payload Template
         _payload = {
-            'personalizations': [{
+            "personalizations": [{
                 # Placeholder
-                'to': [{'email': None}],
+                "to": [{"email": None}],
             }],
-            'from': {
-                'email': self.from_email,
+            "from": {
+                "email": self.from_email,
             },
             # A subject is a requirement, so if none is specified we must
             # set a default with at least 1 character or SendGrid will deny
             # our request
-            'subject': title if title else self.default_empty_subject,
-            'content': [{
-                'type': 'text/plain'
-                if self.notify_format == NotifyFormat.TEXT else 'text/html',
-                'value': body,
+            "subject": title if title else self.default_empty_subject,
+            "content": [{
+                "type": (
+                    "text/plain"
+                    if self.notify_format == NotifyFormat.TEXT
+                    else "text/html"
+                ),
+                "value": body,
             }],
         }
 
@@ -344,41 +364,48 @@ class NotifySendGrid(NotifyBase):
                 if not attachment:
                     # We could not access the attachment
                     self.logger.error(
-                        'Could not access SendGrid attachment {}.'.format(
-                            attachment.url(privacy=True)))
+                        "Could not access SendGrid attachment"
+                        f" {attachment.url(privacy=True)}."
+                    )
                     return False
 
                 try:
                     attachments.append({
                         "content": attachment.base64(),
-                        "filename": attachment.name
-                        if attachment.name else f'file{no:03}.dat',
+                        "filename": (
+                            attachment.name
+                            if attachment.name
+                            else f"file{no:03}.dat"
+                        ),
                         "type": "application/octet-stream",
-                        "disposition": "attachment"
+                        "disposition": "attachment",
                     })
 
                 except exception.AppriseException:
                     # We could not access the attachment
                     self.logger.error(
-                        'Could not access SendGrid attachment {}.'.format(
-                            attachment.url(privacy=True)))
+                        "Could not access SendGrid attachment"
+                        f" {attachment.url(privacy=True)}."
+                    )
                     return False
 
                 self.logger.debug(
-                    'Appending SendGrid attachment {}'.format(
-                        attachment.url(privacy=True)))
+                    "Appending SendGrid attachment"
+                    f" {attachment.url(privacy=True)}"
+                )
 
             # Append our attachments to the payload
             _payload.update({
-                'attachments': attachments,
+                "attachments": attachments,
             })
 
         if self.template:
-            _payload['template_id'] = self.template
+            _payload["template_id"] = self.template
 
             if self.template_data:
-                _payload['personalizations'][0]['dynamic_template_data'] = \
-                    {k: v for k, v in self.template_data.items()}
+                _payload["personalizations"][0]["dynamic_template_data"] = (
+                    dict(self.template_data.items())
+                )
 
         targets = list(self.targets)
         while len(targets) > 0:
@@ -391,24 +418,27 @@ class NotifySendGrid(NotifyBase):
             # below code prepares this by ensuring the target isn't in the cc
             # list or bcc list. It also makes sure the cc list does not contain
             # any of the bcc entries
-            cc = (self.cc - self.bcc - set([target]))
-            bcc = (self.bcc - set([target]))
+            cc = self.cc - self.bcc - {target}
+            bcc = self.bcc - {target}
 
             # Set our target
-            payload['personalizations'][0]['to'][0]['email'] = target
+            payload["personalizations"][0]["to"][0]["email"] = target
 
             if len(cc):
-                payload['personalizations'][0]['cc'] = \
-                    [{'email': email} for email in cc]
+                payload["personalizations"][0]["cc"] = [
+                    {"email": email} for email in cc
+                ]
 
             if len(bcc):
-                payload['personalizations'][0]['bcc'] = \
-                    [{'email': email} for email in bcc]
+                payload["personalizations"][0]["bcc"] = [
+                    {"email": email} for email in bcc
+                ]
 
-            self.logger.debug('SendGrid POST URL: %s (cert_verify=%r)' % (
-                self.notify_url, self.verify_certificate,
-            ))
-            self.logger.debug('SendGrid Payload: %s' % str(payload))
+            self.logger.debug(
+                "SendGrid POST URL:"
+                f" {self.notify_url} (cert_verify={self.verify_certificate!r})"
+            )
+            self.logger.debug(f"SendGrid Payload: {payload!s}")
 
             # Always call throttle before any remote server i/o is made
             self.throttle()
@@ -421,22 +451,25 @@ class NotifySendGrid(NotifyBase):
                     timeout=self.request_timeout,
                 )
                 if r.status_code not in (
-                        requests.codes.ok, requests.codes.accepted):
+                    requests.codes.ok,
+                    requests.codes.accepted,
+                ):
                     # We had a problem
-                    status_str = \
-                        NotifySendGrid.http_response_code_lookup(
-                            r.status_code, SENDGRID_HTTP_ERROR_MAP)
+                    status_str = NotifySendGrid.http_response_code_lookup(
+                        r.status_code, SENDGRID_HTTP_ERROR_MAP
+                    )
 
                     self.logger.warning(
-                        'Failed to send SendGrid notification to {}: '
-                        '{}{}error={}.'.format(
+                        "Failed to send SendGrid notification to {}: "
+                        "{}{}error={}.".format(
                             target,
                             status_str,
-                            ', ' if status_str else '',
-                            r.status_code))
+                            ", " if status_str else "",
+                            r.status_code,
+                        )
+                    )
 
-                    self.logger.debug(
-                        'Response Details:\r\n{}'.format(r.content))
+                    self.logger.debug(f"Response Details:\r\n{r.content}")
 
                     # Mark our failure
                     has_error = True
@@ -444,13 +477,15 @@ class NotifySendGrid(NotifyBase):
 
                 else:
                     self.logger.info(
-                        'Sent SendGrid notification to {}.'.format(target))
+                        f"Sent SendGrid notification to {target}."
+                    )
 
             except requests.RequestException as e:
                 self.logger.warning(
-                    'A Connection error occurred sending SendGrid '
-                    'notification to {}.'.format(target))
-                self.logger.debug('Socket Exception: %s' % str(e))
+                    "A Connection error occurred sending SendGrid "
+                    f"notification to {target}."
+                )
+                self.logger.debug(f"Socket Exception: {e!s}")
 
                 # Mark our failure
                 has_error = True
@@ -460,11 +495,8 @@ class NotifySendGrid(NotifyBase):
 
     @staticmethod
     def parse_url(url):
-        """
-        Parses the URL and returns enough arguments that can allow
-        us to re-instantiate this object.
-
-        """
+        """Parses the URL and returns enough arguments that can allow us to re-
+        instantiate this object."""
 
         results = NotifyBase.parse_url(url)
         if not results:
@@ -480,47 +512,47 @@ class NotifySendGrid(NotifyBase):
         #                 |       |         |
         #              apikey     -from addr-
 
-        if not results.get('user'):
+        if not results.get("user"):
             # An API Key as not properly specified
             return None
 
-        if not results.get('password'):
+        if not results.get("password"):
             # A From Email was not correctly specified
             return None
 
         # Prepare our API Key
-        results['apikey'] = NotifySendGrid.unquote(results['user'])
+        results["apikey"] = NotifySendGrid.unquote(results["user"])
 
         # Prepare our From Email Address
-        results['from_email'] = '{}@{}'.format(
-            NotifySendGrid.unquote(results['password']),
-            NotifySendGrid.unquote(results['host']),
+        results["from_email"] = "{}@{}".format(
+            NotifySendGrid.unquote(results["password"]),
+            NotifySendGrid.unquote(results["host"]),
         )
 
         # Acquire our targets
-        results['targets'] = NotifySendGrid.split_path(results['fullpath'])
+        results["targets"] = NotifySendGrid.split_path(results["fullpath"])
 
         # The 'to' makes it easier to use yaml configuration
-        if 'to' in results['qsd'] and len(results['qsd']['to']):
-            results['targets'] += \
-                NotifySendGrid.parse_list(results['qsd']['to'])
+        if "to" in results["qsd"] and len(results["qsd"]["to"]):
+            results["targets"] += NotifySendGrid.parse_list(
+                results["qsd"]["to"]
+            )
 
         # Handle Carbon Copy Addresses
-        if 'cc' in results['qsd'] and len(results['qsd']['cc']):
-            results['cc'] = \
-                NotifySendGrid.parse_list(results['qsd']['cc'])
+        if "cc" in results["qsd"] and len(results["qsd"]["cc"]):
+            results["cc"] = NotifySendGrid.parse_list(results["qsd"]["cc"])
 
         # Handle Blind Carbon Copy Addresses
-        if 'bcc' in results['qsd'] and len(results['qsd']['bcc']):
-            results['bcc'] = \
-                NotifySendGrid.parse_list(results['qsd']['bcc'])
+        if "bcc" in results["qsd"] and len(results["qsd"]["bcc"]):
+            results["bcc"] = NotifySendGrid.parse_list(results["qsd"]["bcc"])
 
         # Handle Blind Carbon Copy Addresses
-        if 'template' in results['qsd'] and len(results['qsd']['template']):
-            results['template'] = \
-                NotifySendGrid.unquote(results['qsd']['template'])
+        if "template" in results["qsd"] and len(results["qsd"]["template"]):
+            results["template"] = NotifySendGrid.unquote(
+                results["qsd"]["template"]
+            )
 
         # Add any template substitutions
-        results['template_data'] = results['qsd+']
+        results["template_data"] = results["qsd+"]
 
         return results
